from typing import overload, Optional, Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..Datum.DatumCsys import DatumCsys
from ..Mesh.MeshEdge import MeshEdge
from ..Mesh.MeshElement import MeshElement
from ..Mesh.MeshFace import MeshFace
from ..Mesh.MeshNode import MeshNode
from ..Part.PartBase import PartBase
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, OFF, ON, OUTWARD, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Node(MeshNode):
    ...


class MeshEditPart(PartBase):
    """The following commands operate on Part objects. For more information about the Part
    object, see Part object.

    .. note::
        This object can be accessed by::

            import meshEdit
    """

    @abaqus_method_doc
    def adjustMidsideNode(self, cornerNodes: Sequence[Node], parameter: float):
        """This method is used to adjust the midside node of second-order elements of an orphan
        mesh part.

        Parameters
        ----------
        cornerNodes
            A sequence of Node objects specifying the nodes towards which connected midside nodes
            will be biased.
        parameter
            A Float specifying the normalized distance along the edge of the midside nodes. Possible
            values are 0.0 ≤ **parameter** ≤ 1.0, where 0.0 specifies the position of the corner
            node. The default value is 0.5.
        """
        ...

    @abaqus_method_doc
    def cleanMesh(
        self,
        mergeTolerance: float,
        growEdges: Boolean = OFF,
        elements: str = "",
        refEdge: str = "",
        thicknessDir: Optional[float] = None,
        moveLayers: Boolean = False,
    ):
        """This method is used to collapse short element edges and delete collapsed elements, or
        grow short element edges, on an orphan mesh part composed of linear elements.

        Parameters
        ----------
        mergeTolerance
            A Float specifying the edge length tolerance. During the operation, edges shorter than
            the given tolerance will be collapsed, or grown to the specified length.
        growEdges
            A Boolean specifying whether short element edges will be grown to the specified
            tolerance. Default is False, meaning short edges will be collapsed.
        elements
            The elements to consider as the domain for the operation. By default all elements on the
            part are considered. The elements may be given as a MeshElementArray, a list of
            MeshElement objects, a Set, or a list of Set objects.
        refEdge
            A MeshEdge specifying a reference edge to indicate a topological direction in a
            structured mesh that will limit which edges within the element domain are considered.
            That is, only edges that are found to be topologically parallel to the given reference
            edge will be considered by the operation. By default all edges of the element domain are
            considered, unless **thicknessDir** is specified, in which case the operation will attempt
            to determine the topological edges from the thickness direction.
        thicknessDir
            A tuple of two or three Floats indicating a vector along which element edge lengths will
            be measured.
        moveLayers
            A Boolean indicating whether element edges will be all grown in the direction of the
            thickness vector specified by **thicknessDir**. This argument is ignored unless
            **growEdges** is True and **thicknessDir** is provided. When this argument is True the
            growth of any given element edge will no longer be constrained by short edges on
            neighboring elements, but elements could move from their original positions in cases
            where there are multiple adjacent layers of thin elements. The default value is False.
        """
        ...

    @abaqus_method_doc
    def collapseMeshEdge(self, edge: str, collapseMethod: Literal[C.FORWARD, C.REVERSE, C.AVERAGE]):
        """This method collapses an edge of a quadrilateral or triangular element of an orphan mesh
        part or part instance.

        Parameters
        ----------
        edge
            A single MeshEdge object specifying the element edge to collapse.
        collapseMethod
            A SymbolicConstant specifying the method used to collapse the edge. Possible values are
            FORWARD, REVERSE, and AVERAGE.
        """
        ...

    @abaqus_method_doc
    def combineElement(self, elements: tuple):
        """This method combines two triangular elements of an orphan mesh part or an Abaqus native
        mesh.

        Parameters
        ----------
        elements
            A sequence of triangular MeshElement objects specifying the elements to combine.
        """
        ...

    @abaqus_method_doc
    def convertSolidMeshToShell(self):
        """This method removes all solid elements from an orphan mesh part and creates triangular
        or quadrilateral shell elements along their outer faces.
        """
        ...

    @abaqus_method_doc
    def deleteElement(self, elements: Sequence[MeshElement], deleteUnreferencedNodes: Boolean = OFF):
        """This method deletes the given elements from an orphan mesh part or an Abaqus native
        mesh. If the elements belong to an Abaqus native mesh then the elements must have been
        generated using the bottom-up meshing technique.

        Parameters
        ----------
        elements
            A sequence of MeshElement objects or a Set object containing elements.
        deleteUnreferencedNodes
            A Boolean specifying whether to delete all those associated nodes that become
            unreferenced after the given elements are deleted. The default value is OFF.
        """
        ...

    @abaqus_method_doc
    def deleteNode(self, nodes: Sequence[MeshNode], deleteUnreferencedNodes: Boolean = OFF):
        """This method deletes the given nodes from an orphan mesh part.

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects or a Set object containing nodes.
        deleteUnreferencedNodes
            A Boolean specifying whether to delete all those associated nodes that become
            unreferenced after the given nodes and the connected elements are deleted. The default
            value is OFF.
        """
        ...

    @abaqus_method_doc
    def editNode(
        self,
        nodes: Sequence[MeshNode],
        coordinate1: Optional[float] = None,
        coordinate2: Optional[float] = None,
        coordinate3: Optional[float] = None,
        coordinates: Sequence[float] = (),
        offset1: Optional[float] = None,
        offset2: Optional[float] = None,
        offset3: Optional[float] = None,
        localCsys: Optional[DatumCsys] = None,
        projectToGeometry: Boolean = ON,
    ):
        """This method changes the coordinates of the given nodes on an orphan mesh part or on an
        Abaqus native mesh.

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects or a Set object containing nodes.
        coordinate1
            A Float specifying the value of the first coordinate. If **coordinate1** and **offset1** are
            unspecified, the existing value does not change.
        coordinate2
            A Float specifying the value of the second coordinate. If **coordinate2** and **offset2**
            are unspecified, the existing value does not change.
        coordinate3
            A Float specifying the value of the third coordinate. If **coordinate3** and **offset3** are
            unspecified, the existing value does not change.
        coordinates
            A sequence of three-dimensional coordinate tuples specifying the coordinates for each of
            the given nodes. When specified, the number of coordinate tuples must match the number
            of given nodes, and be ordered to correspond to the given nodes in **ascending order**
            according to index. Furthermore, **coordinate1**, **coordinate2**, **coordinate3**, **offset1**,
            **offset2**, or **offset3** may not be specified.
        offset1
            A Float specifying an offset to apply to the value of the first coordinate of the
            specified nodes.
        offset2
            A Float specifying an offset to apply to the value of the second coordinate of the
            specified nodes.
        offset3
            A Float specifying an offset to apply to the value of the third coordinate of the
            specified nodes.
        localCsys
            A DatumCsys object specifying the local coordinate system. If unspecified, the global
            coordinate system will be used.
        projectToGeometry
            A Boolean specifying whether to project nodes back to their original geometry. For
            example, if a node is on a face, this method first positions the node at the new
            location and then projects it back to the original face. The default value is ON.

        Raises
        ------
        A coordinate and an offset may not both be specified for the same coordinate component
        """
        ...

    @abaqus_method_doc
    def projectNode(self, nodes: Sequence[MeshNode], projectionReference: str):
        """This method projects the given nodes onto a mesh entity, geometric entity, or a datum
        object. The nodes may belong to an orphan mesh part or to an Abaqus native mesh.

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects to be projected.
        projectionReference
            An object specifying the target for the node projection operation. The
            **projectionReference** can be any one of the following objects: MeshNode, MeshEdge,
            MeshFace, ConstrainedSketchVertex, Edge, Face, DatumPoint, DatumAxis, or DatumPlane.
        """
        ...

    @abaqus_method_doc
    def generateMesh(self, elemShape: Optional[SymbolicConstant] = None):
        """This method generates a new mesh on an orphan mesh part based on the original mesh.

        Parameters
        ----------
        elemShape
            A SymbolicConstant specifying the element shape to be used for meshing. Possible values
            are:TRIRefine a planar triangular mesh and replace it with a new one. If no element
            sizes are attached, the new mesh will be governed by the sizes of the boundary edges in
            the old mesh. TETCreate a tetrahedral mesh from a closed shell of triangular elements.
        """
        ...

    @abaqus_method_doc
    def generateMeshByOffset(
        self,
        region: Region,
        meshType: str,
        totalThickness: float,
        distanceBetweenLayers: float,
        numLayers: int,
        offsetDirection: str = OUTWARD,
        initialOffset: float = 0.0,
        shareNodes: str = False,
        deleteBaseElements: Boolean = False,
        constantThicknessCorners: Boolean = False,
        extendElementSets: Boolean = False,
    ):
        """This method generates a solid or shell mesh from an orphan mesh surface by generating
        layers of elements that propagate out normal to the surface boundary.

        Parameters
        ----------
        region
            A Region object specifying the domain to be offset.
        meshType
            A Symbolic Constant specifying the type of mesh to be generated. Possible values are
            SOLID or SHELL.
        totalThickness
            A Float specifying the total thickness of the solid layers. This argument applies only
            when **meshType** = SOLID.
        distanceBetweenLayers
            A Float specifying the distance between shell layers. This argument applies only when
            **meshType** = SHELL.
        numLayers
            An Int specifying the number of element layers to be generated.
        offsetDirection
            A Symbolic Constant specifying the direction of the offset. This argument is required
            only when the given region relates to a shell mesh. Possible values are OUTWARD, INWARD,
            and BOTH. The default value is OUTWARD.
        initialOffset
            A Float specifying the magnitude of the initial offset. The default value is zero.
        shareNodes
            Boolean specifying whether the first layer of nodes should be shared with nodes on the
            base surface. The default value is False.
        deleteBaseElements
            A Boolean specifying whether to delete the shell elements after the offset layers are
            generated. The default value is False. This argument applies only when **meshType** = SHELL.
        constantThicknessCorners
            A Boolean specifying whether to use element-based thickness or nodal-based thickness.
            The default value is False.
        extendElementSets
            A Boolean specifying whether existing element sets that include base elements will be
            extended to include corresponding offset elements. The default value is False.
        """
        ...

    @abaqus_method_doc
    def mergeElement(self, edge: str, elements: str):
        """Merge a selection of elements arranged in layers on an orphan mesh part into a single
        layer.

        Parameters
        ----------
        edge
            A MeshEdge of one of the specified elements that serves as a reference edge to indicate
            the topological direction for merging elements. All specified elements must be reachable
            by topological navigation from this element edge, and the topological direction must be
            unambiguous.
        elements
            A MeshElementArray, a list of MeshElement objects, a Set, or a list of Set objects
            containing the elements to be included in the merge operation.
        """
        ...

    @overload
    @abaqus_method_doc
    def mergeNodes(
        self,
        nodes: Sequence[Node],
        tolerance: Optional[float] = None,
        removeDuplicateElements: Boolean = True,
        keepHighLabels: Boolean = False,
    ):
        """Merge the nodes of an orphan mesh part, or nodes that were generated using the bottom-up
        meshing technique.

        Parameters
        ----------
        nodes
            A sequence of Node objects specifying the nodes to merge.
        tolerance
            A Float specifying the maximum distance between nodes that will be merged to a single
            node. The location of the new node is the average position of the merged nodes. The
            default value is 10⁻⁶.
        removeDuplicateElements
            A Boolean specifying whether elements with the same connectivity after the merge will
            merged into a single element. The default value is True.
        keepHighLabels
            A Boolean specifying which node labels will remain after nodes are merged. If True then
            the highest node labels are kept; when False the lowest node labels are kept. The
            default value is False. This argument applies only to merging of orphan mesh nodes.
        """
        ...

    @overload
    @abaqus_method_doc
    def mergeNodes(
        self,
        node1: MeshNode,
        node2: MeshNode,
        removeDuplicateElements: Boolean = True,
        keepHighLabels: Boolean = False,
    ):
        """Merge two nodes of an orphan mesh part or an Abaqus native mesh. If the nodes belong to
        an Abaqus native mesh then at least one of the two nodes must have been generated using
        the bottom-up meshing technique.

        Parameters
        ----------
        node1
            A MeshNode object specifying the first node to merge.
        node2
            A MeshNode object specifying the second node to merge.
        removeDuplicateElements
            A Boolean specifying whether elements with the same connectivity after the merge will
            merged into a single element. The default value is True.
        keepHighLabels
            A Boolean specifying which node label will remain after nodes are merged. If True then
            the higher node label is kept; when False the lower node label is kept. The default
            value is False. This argument applies only to merging of orphan mesh nodes.
        """
        ...

    @abaqus_method_doc
    def mergeNodes(self, *args, **kwargs):
        ...

    def orientElements(self, pickedElements: Sequence[MeshElement], referenceRegion: MeshFace):
        """This method orients the stack direction of elements in a continuum shell or gasket mesh.

        Parameters
        ----------
        pickedElements
            A sequence of MeshElement objects specifying the elements to orient.
        referenceRegion
            A MeshFace object specifying a reference top face that indicates the desired
            orientation.
        """
        ...

    @abaqus_method_doc
    def removeElementSize(self):
        """This method removes the global element size from an orphan mesh part."""
        ...

    @abaqus_method_doc
    def renumberElement(
        self,
        elements: tuple = (),
        startLabel: Optional[int] = None,
        increment: Optional[int] = None,
        offset: Optional[int] = None,
        labels: str = "",
    ):
        """This method assigns new labels to orphan mesh elements.

        Parameters
        ----------
        elements
            A MeshElementArray or a tuple or list of MeshElement objects, or a Set containing
            elements to be renumbered. If unspecified, all elements in the part will be renumbered.
        startLabel
            A positive Int specifying the new label for the first element in **elements**.
        increment
            A positive Int specifying the increment used for computation of new labels for all
            consecutive elements in **elements**.
        offset
            An Int by which existing labels of the specified elements will be offset.
        labels
            A list of labels for the specified elements. The length of this list must match the
            number of specified elements.

        Raises
        ------
        Error: Renumbering can be applied to orphan mesh parts only
            Renumbering is attempted on a native part
        Error: Either startLabel and increment or offset must be specified
            Renumbering data is specified incorrectly
        Error: Specified data will result in invalid labels
            Renumbering will result in invalid labels
        Error: Specified data will result in conflicting labels
            Renumbering will result in conflicting labels
        """
        ...

    @abaqus_method_doc
    def renumberNode(
        self,
        nodes: tuple = (),
        startLabel: Optional[int] = None,
        increment: Optional[int] = None,
        offset: Optional[int] = None,
        labels: str = "",
    ):
        """This method assigns new labels to orphan mesh nodes.

        Parameters
        ----------
        nodes
            A MeshNodeArray or a tuple or list of MeshNode objects, or a Set containing nodes to be
            renumbered. If unspecified, all nodes in the part will be renumbered.
        startLabel
            A positive Int specifying the new label for the first node in **nodes**.
        increment
            A positive Int specifying the increment used for computation of new labels for all
            consecutive nodes in **nodes**.
        offset
            An Int by which existing labels of the specified nodes will be offset.
        labels
            A list of labels for the specified nodes. The length of this list must match the number
            of specified nodes.

        Raises
        ------
        Error: Renumbering can be applied to orphan mesh parts only
            Renumbering is attempted on a native part.
        Error: Either startLabel and increment or offset must be specified
            Renumbering data is specified incorrectly.
        Error: Specified data will result in invalid labels
            Renumbering will result in invalid labels.
        Error: Specified data will result in conflicting labels
            Renumbering will result in conflicting labels.
        """
        ...

    @abaqus_method_doc
    def setElementSize(self, size: float):
        """This method sets the global element size for an orphan mesh part.

        Parameters
        ----------
        size
            A Float specifying the desired element size.
        """
        ...

    @abaqus_method_doc
    def splitElement(self, elements: tuple):
        """This method splits quadrilateral elements of an orphan mesh part or a Abaqus native mesh
        into triangular elements.

        Parameters
        ----------
        elements
            A sequence of quadrilateral MeshElement objects specifying the elements to split. Each
            quadrilateral element is split into two triangular elements by the shorter diagonal.
        """
        ...

    @abaqus_method_doc
    def splitMeshEdge(self, edge: str, parameter: float = 0):
        """This method splits an edge of a quadrilateral or triangular element of an orphan mesh
        part or an Abaqus native mesh.

        Parameters
        ----------
        edge
            A single MeshEdge object specifying the element edge to split.
        parameter
            A Float specifying the normalized distance along **edge** at which to split. Possible
            values are 0.0 < **parameter** < 1.0. The default value is 0.5.
        """
        ...

    @abaqus_method_doc
    def subdivideElement(
        self,
        elements: str = "",
        divisionNumber: int = 2,
        face: Optional[MeshFace] = None,
        edge: Optional[MeshEdge] = None,
    ):
        """Subdivide a selection of elements on an orphan mesh part in one or more directions.

        Parameters
        ----------
        elements
            A MeshElementArray, a list of MeshElement objects, a Set, or a list of Set objects
            containing the elements to be subdivided. By default all the elements of the part are
            subdivided.
        divisionNumber
            An Int specifying the number of resulting elements for each input element in each
            direction of the subdivision. If **face** or **edge** are not specified, elements will be
            subdivided according to this number in all possible directions. Must be greater than
            one. Default is 2.
        face
            A MeshFace object that serves as a reference for indicating two topological directions
            for the subdivision operation. Must be a face of one of the specified elements, and all
            specified elements must be reachable by topological navigation from this element face.
            May not be combined with **edge**.
        edge
            A MeshEdge object that serves as a reference for indicating a single topological
            direction for the subdivision operation. Must be an edge of one of the specified
            elements, and all specified elements must be reachable by topological navigation from
            this element edge. May not be combined with **face**.
        """
        ...

    @abaqus_method_doc
    def swapMeshEdge(self, edge: str):
        """This method swaps the diagonal of two adjacent triangular elements of an orphan mesh
        part or an Abaqus native mesh.

        Parameters
        ----------
        edge
            A single MeshEdge object specifying the element edge to swap.
        """
        ...

    @abaqus_method_doc
    def wrapMesh(self, radius: float):
        """This method wraps a planar orphan mesh part about the **Z**-axis.

        Parameters
        ----------
        radius
            A Float specifying the radius of the cylinder about which the part is to be wrapped. The
            wrapping procedure will relocate a node at point (xx, yy) on the planar mesh to
            (x,θ,zx,θ,z), where rr is the specified radius, θθ = xrxr, and zz=yy.
        """
        ...

    @abaqus_method_doc
    def redoMeshEdit(self):
        """This method executes the edit mesh or the bottom-up meshing operation most recently
        undone by the undoMeshEdit method on an part. A redo action must be currently available
        for the part. This implies that the user must have executed the undoMeshEdit method on
        the part and that the user has not subsequently executed any further edit mesh commands
        on the assembly. It also implies that the user provided a sufficient cache allowance to
        store the undo operation.
        """
        ...

    @abaqus_method_doc
    def undoMeshEdit(self):
        """This method undoes the most recent edit mesh or the bottom-up meshing operation on a
        part and restores the mesh to its previous state. An edit mesh undo action must be
        available for the part. This implies that prior to executing an edit mesh command on the
        part, the user enabled edit mesh undo with a sufficient cache allowance to store the
        edit mesh operation.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..UtilityAndView.abaqusConstants import Boolean, COLOR, OFF, ON, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .._OptionsBase import _OptionsBase


@abaqus_class_doc
class PrintOptions(_OptionsBase):
    """The PrintOptions object stores the common settings that Abaqus uses for all print
    methods. The PrintOptions object has no constructor. Abaqus creates the **printOptions**
    member when a session is started.

    .. note::
        This object can be accessed by::

            session.printOptions
    """

    #: A SymbolicConstant specifying how the image is rendered. Possible values are
    #: BLACK_AND_WHITE, GREYSCALE, and COLOR. The default value is COLOR.
    rendition: SymbolicConstant = COLOR

    #: A Boolean specifying whether the output includes the viewport border and title. The
    #: default value is ON.
    vpDecorations: Boolean = ON

    #: A Boolean specifying whether the output includes viewport backgrounds. The default value is
    #: OFF.
    vpBackground: Boolean = OFF

    #: A Boolean specifying whether the output includes the view compass. The default value is OFF.
    compass: Boolean = OFF

    #: A String specifying the default print command that will be used by the printToPrinter method
    #: if no **printCommand** argument is provided. The default value is "lpr".
    printCommand: str = "lpr"

    #: A Boolean specifying whether to delete the temporary files created when an image is printed.
    #: The default value is ON. You should set the **deleteTemporaryFiles** argument to False if your
    #: printer does not support print spooling.
    deleteTemporaryFiles: Boolean = ON

    #: A Boolean specifying whether the raster image printed is reduced from true color to 256 colors
    #: to reduce file size. The quality of the image will be compromised. The default value is OFF.
    reduceColors: Boolean = OFF

    @abaqus_method_doc
    def setValues(
        self,
        *,
        rendition: Literal[C.COLOR, C.GREYSCALE, C.BLACK_AND_WHITE] = COLOR,
        vpDecorations: Boolean = ON,
        vpBackground: Boolean = OFF,
        compass: Boolean = OFF,
        printCommand: str = "lpr",
        deleteTemporaryFiles: Boolean = ON,
        reduceColors: Boolean = OFF,
    ):
        """This method modifies the PrintOptions object.

        Parameters
        ----------
        rendition
            A SymbolicConstant specifying how the image is rendered. Possible values are
            BLACK_AND_WHITE, GREYSCALE, and COLOR. The default value is COLOR.
        vpDecorations
            A Boolean specifying whether the output includes the viewport border and title. The
            default value is ON.
        vpBackground
            A Boolean specifying whether the output includes viewport backgrounds. The default value
            is OFF.
        compass
            A Boolean specifying whether the output includes the view compass The default value is
            OFF.
        printCommand
            A String specifying the default print command that will be used by the printToPrinter
            method if no **printCommand** argument is provided. The default value is "lpr".
        deleteTemporaryFiles
            A Boolean specifying whether to delete the temporary files created when an image is
            printed. The default value is ON. You should set the **deleteTemporaryFiles** argument to
            False if your printer does not support print spooling.
        reduceColors
            A Boolean specifying whether the raster image printed is reduced from true color to 256
            colors to reduce file size. The quality of the image will be compromised. The default
            value is OFF.
        """
        super().setValues(
            rendition=rendition,
            vpDecorations=vpDecorations,
            vpBackground=vpBackground,
            compass=compass,
            printCommand=printCommand,
            deleteTemporaryFiles=deleteTemporaryFiles,
            reduceColors=reduceColors,
        )



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..Job.JobMdb import JobMdb
from ..Model.Model import Model
from ..Part.AcisMdb import AcisMdb
from ..UtilityAndView.abaqusConstants import Boolean, NOT_SET, ON, STANDARD_EXPLICIT
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Mdb(AcisMdb, JobMdb):
    """The Mdb object is the high-level Abaqus model database. A model database stores models
    and analysis controls.

    .. note::
        This object can be accessed by::

            mdb
    """

    @abaqus_method_doc
    def Model(
        self,
        name: str,
        description: str = "",
        stefanBoltzmann: Optional[float] = None,
        absoluteZero: Optional[float] = None,
        waveFormulation: Literal[C.SCATTERED, C.NOT_SET, C.TOTAL] = NOT_SET,
        modelType: Literal[C.STANDARD_EXPLICIT, C.ELECTROMAGNETIC] = STANDARD_EXPLICIT,
        universalGas: Optional[float] = None,
        copyConstraints: Boolean = ON,
        copyConnectors: Boolean = ON,
        copyInteractions: Boolean = ON,
    ) -> Model:
        """This method creates a Model object.

        .. note::
            This function can be accessed by::

                mdb.Model

        Parameters
        ----------
        name
            A String specifying the repository key.
        description
            A String specifying the purpose and contents of the Model object. The default value is
            an empty string.
        stefanBoltzmann
            None or a Float specifying the Stefan-Boltzmann constant. The default value is None.
        absoluteZero
            None or a Float specifying the absolute zero constant. The default value is None.
        waveFormulation
            A SymbolicConstant specifying the type of incident wave formulation to be used in
            acoustic problems. Possible values are NOT_SET, SCATTERED, and TOTAL. The default value
            is NOT_SET.
        modelType
            A SymbolicConstant specifying the analysis model type. Possible values are
            STANDARD_EXPLICIT and ELECTROMAGNETIC. The default is STANDARD_EXPLICIT.
        universalGas
            None or a Float specifying the universal gas constant. The default value is None.
        copyConstraints
            A boolean specifying whether to copy the constraints created in the model to the model
            that instances this model. The default value is ON.
        copyConnectors
            A boolean specifying whether to copy the connectors created in the model to the model
            that instances this model. The default value is ON.
        copyInteractions
            A boolean specifying whether to copy the interactions created in the model to the model
            that instances this model. The default value is ON.

        Returns
        -------
        model: Model
            A Model object
        """
        self.models[name] = model = Model(
            name,
            description,
            stefanBoltzmann,
            absoluteZero,
            waveFormulation,
            modelType,
            universalGas,
            copyConstraints,
            copyConnectors,
            copyInteractions,
        )
        return model



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..Region.Region import Region


@abaqus_class_doc
class IMARegion:
    """A IMARegion is an object used to define material instance name volume fractions for the
    MaterialAssignment predefined field.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].predefinedFields[name].assignmentList
    """

    #: A Region object specifying the sub-region of the selected part instance to which the
    #: volume fractions will be applied.
    region: Region = Region()

    #: A tuple of Floats specifying the volume fractions, per material instance name. The
    #: length of the tuple corresponds to the number of material instance names, as established
    #: by the assigned Eulerian section.
    fractionList: Optional[float] = None



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ....UtilityAndView.abaqusConstants import Boolean, OFF, STRAIN
from ....UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ConcreteTensionDamage:
    r"""The ConcreteTensionDamage object specifies hardening for the concrete damaged plasticity
    model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].concreteDamagedPlasticity.concreteTensionDamage
            import odbMaterial
            session.odbs[name].materials[name].concreteDamagedPlasticity.concreteTensionDamage

        The table data for this object are:

        - If **type** = STRAIN, the table data specify the following:

            - Tensile damage variable, :math:`d_{t}`.
            - Direct cracking strain, :math:`\epsilon_{t}^{c k}`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = DISPLACEMENT, the table data specify the following:

            - Tensile damage variable, :math:`d_{t}`
            - Direct cracking displacement, :math:`u_{t}^{c k}`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - CONCRETE TENSION DAMAGE
    """

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        compressionRecovery: float = 1,
        type: Literal[C.STRAIN, C.DISPLACEMENT] = STRAIN,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a ConcreteTensionDamage object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].concreteDamagedPlasticity.ConcreteTensionDamage.ConcreteTensionDamageaterials[name].concreteDamagedPlasticity.ConcreteTensionDamage

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        compressionRecovery
            A Float specifying the value of the stiffness recovery factor, :math:`w_c`, that determines the
            amount of compression stiffness that is recovered as loading changes from tension to
            compression. The default value is 1.0.
        type
            A SymbolicConstant specifying the type of tensile damage data. Set **type** = STRAIN to
            specify the tensile damage variable as a function of cracking strain. Set
            **type** = DISPLACEMENT to specify the tensile damage variable as a function of cracking
            displacement. Possible values are STRAIN and DISPLACEMENT. The default value is STRAIN.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        ConcreteTensionDamage
            A ConcreteTensionDamage object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ConcreteTensionDamage object.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .Annotation import Annotation
from ..Canvas.ViewportBase import ViewportBase


@abaqus_class_doc
class AnnotationViewport(ViewportBase):
    """The following commands operate on Viewport objects. For more information about the
    Viewport object, see Viewport object.

    .. note::
        This object can be accessed by::

            import annotationToolset
    """

    @abaqus_method_doc
    def plotAnnotation(self, annotation: Annotation, index: int = 0):
        """This method plots an Annotation object in aViewport.

        Parameters
        ----------
        annotation
            An Annotation object to plot.
        index
            An Int specifying the index of the Annotation object in the sequence of annotations to
            plot. The default value is zero.
        """
        # TODO: implement this method
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ....UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class ClayHardening:
    """The ClayHardening object specifies hardening for the clay plasticity model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].clayPlasticity.clayHardening
            import odbMaterial
            session.odbs[name].materials[name].clayPlasticity.clayHardening

        The table data for this object are:

        - The hydrostatic pressure stress at yield, :math:`p_c`.
        - The absolute value of the corresponding volumetric Plastic strain.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - CLAY HARDENING
    """

    @abaqus_method_doc
    def __init__(self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0):
        """This method creates a ClayHardening object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].clayPlasticity.ClayHardening
                session.odbs[name].materials[name].clayPlasticity.ClayHardening

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        ClayHardening
            A ClayHardening object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ClayHardening object.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .ConstrainedSketchConstraint import ConstrainedSketchConstraint
from ..ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)


@abaqus_class_doc
class TangentConstraint(ConstrainedSketchConstraint):
    @abaqus_method_doc
    def __init__(self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry):
        """This method creates a tangent constraint. This constraint applies to different types of
        ConstrainedSketchGeometry objects and constrains them to remain tangential.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].TangentConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first object.
        entity2
            A ConstrainedSketchGeometry object specifying the second object.

        Returns
        -------
        ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object.

        """
        ...



"""
Run this file by `python update-suggested-version.py "20xx.x.x"` to update the version in
README.md, README-zh-cn.md, and docs/source/getting_started.rst.
"""
import os
import re
import sys


current_version = sys.argv[1]
if current_version.startswith("v"):
    current_version = current_version[1:]
os.chdir(os.path.dirname(os.path.abspath(__file__)))
os.chdir("..")

for file in ['docs/source/getting_started.md']:
    with open(file, 'r', encoding='utf-8') as f:
        content = f.read()
    
    pattern = r'abqpy(=+?)20\d\d\.\d+?\.\d+'
    repl = f'abqpy\g<1>{current_version}'
    content = re.sub(pattern, repl, content)
    
    with open(file, 'w', encoding='utf-8') as f:
        f.write(content)



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .Interaction import Interaction
from ..Region.Region import Region


@abaqus_class_doc
class ElasticFoundation(Interaction):
    """The ElasticFoundation object defines a mechanical foundation.
    The ElasticFoundation object is derived from the Interaction object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name]

        The corresponding analysis keywords are:

        - FOUNDATION
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the step in which the ElasticFoundation object is
    #: created. **createStepName** must be set to 'Initial'.
    createStepName: str

    #: A Region object specifying the surface to which the foundation applies.
    surface: Region

    #: A Float specifying the foundation stiffness per area (or per length for beams).
    stiffness: float

    @abaqus_method_doc
    def __init__(self, name: str, createStepName: str, surface: Region, stiffness: float):
        """This method creates an ElasticFoundation object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ElasticFoundation

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the ElasticFoundation object is
            created. **createStepName** must be set to 'Initial'.
        surface
            A Region object specifying the surface to which the foundation applies.
        stiffness
            A Float specifying the foundation stiffness per area (or per length for beams).

        Returns
        -------
        ElasticFoundation
            An ElasticFoundation object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the data for an existing ElasticFoundation object in the step where
        it is created.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str, stiffness: Optional[float] = None):
        """This method modifies the propagating data of an existing ElasticFoundation object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the interaction is modified.
        stiffness
            A Float specifying the foundation stiffness per area (or per length for beams).
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .Leaf import Leaf
from ..UtilityAndView.abaqusConstants import DEFAULT_MODEL, SymbolicConstant


@abaqus_class_doc
class LeafFromOdbNodePick(Leaf):
    """The LeafFromOdbNodePick object can be used whenever a Leaf object is expected as an
    argument. Leaf objects are used to specify the items in a display group. Leaf objects
    are constructed as temporary objects, which are then used as arguments to DisplayGroup
    commands.
    The LeafFromOdbNodePick object is derived from the Leaf object.

    .. note::
        This object can be accessed by::

            import displayGroupOdbToolset
    """

    #: A SymbolicConstant specifying the leaf type. Possible values are EMPTY_LEAF,
    #: DEFAULT_MODEL, ALL_ELEMENTS, ALL_NODES, and ALL_SURFACES.
    leafType: Optional[SymbolicConstant] = None

    @abaqus_method_doc
    def __init__(self, nodePick: tuple):
        """This method creates a Leaf object from a tuple containing machine readable, compact
        strings defining the nodes picked for each part instance. Leaf objects specify the items
        in a display group.

        .. note::
            This function can be accessed by::

                LeafFromOdbNodePick

        Parameters
        ----------
        nodePick
            A sequence of tuples of the form [part name, entity count, machine readable pick
            strings].

        Returns
        -------
        LeafFromOdbNodePick
            A LeafFromOdbNodePick object.
        """
        super().__init__(DEFAULT_MODEL)



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .Constraint import Constraint
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class AdjustPoints(Constraint):
    """The AdjustPoints constraint object is used to adjust points (nodes) to a surface.
    The AdjustPoints object is derived from the ConstrainedSketchConstraint object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].constraints[name]

        The corresponding analysis keywords are:

        - ADJUST
    """

    #: A Boolean specifying whether the constraint is suppressed or not. The default value is
    #: OFF.
    suppressed: Boolean = OFF

    #: A String specifying the constraint repository key.
    name: str

    #: A Region object specifying the surface to which the **controlPoints** are adjusted.
    surface: Region

    #: A Region object specifying the constraint control points.
    controlPoints: Region

    @abaqus_method_doc
    def __init__(self, name: str, surface: Region, controlPoints: Region):
        """This method creates an AdjustPoints object.

        .. note::
            This function can be accessed by::

                mdb.models[name].AdjustPoints

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        surface
            A Region object specifying the surface to which the **controlPoints** are adjusted.
        controlPoints
            A Region object specifying the constraint control points.

        Returns
        -------
        AdjustPoints
            An AdjustPoints object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the AdjustPoints object."""
        ...
